/**
 * @file 01_Send.c
 * @author your name (you@domain.com)
 * @brief
 * @version 0.1
 * @date 2023-09-11
 *
 * @copyright Copyright (c) 2023
 *
 */
#include "debug.h"

#define USE_TX_INTERRUPT 0
#define CAN_DEVICE_STATE_ACTIVE 0
#define CAN_DEVICE_STATE_PASSTIVE 1
#define CAN_DEVICE_STATE_BUSOFF 2

uint8_t DeviceState;

/**
 * @brief 串口调试初始化
 *
 * - 更新系统时钟
 * - 调试延时函数初始化
 * - 调试串口波特率配置
 * - 打印系统时钟频率
 *
 * @param baudrate 串口输出波特率
 */
static void System_debug_init(uint32_t baudrate) {
    RCC_ClocksTypeDef clocks;

    SystemCoreClockUpdate();
    Delay_Init();
    USART_Printf_Init(baudrate);
    RCC_GetClocksFreq(&clocks);

    printf("ChipID: %#10x\r\n", DBGMCU_GetCHIPID()); // 获取芯片型号标识
    printf("DeviceID: %d\r\n", DBGMCU_GetDEVID());   // 获取芯片串号
    printf("Core Frequency: %dHz\r\n", SystemCoreClock);
    printf("AHB  Frequency: %dHz\r\n", clocks.HCLK_Frequency);
    printf("APB1 Frequency: %dHz\r\n", clocks.PCLK1_Frequency);
    printf("APB2 Frequency: %dHz\r\n", clocks.PCLK2_Frequency);
    printf("ADC  Frequency: %dHz\r\n", clocks.ADCCLK_Frequency);
}

/**
 * @brief CAN1 初始化
 *
 * @param brp 外设时钟分频值,取值范围: 0-1023
 * @param bs1 传输时间段1,取值范围: 0-15
 *        |   可选参数   | CAN BS1时间量子          |
 *        | :----------: | :---------------------: |
 *        | CAN_BS1_1tq  |           1             |
 *        | CAN_BS1_2tq  |           2             |
 *        | CAN_BS1_3tq  |           3             |
 *        | CAN_BS1_4tq  |           4             |
 *        | CAN_BS1_5tq  |           5             |
 *        | CAN_BS1_6tq  |           6             |
 *        | CAN_BS1_7tq  |           7             |
 *        | CAN_BS1_8tq  |           8             |
 *        | CAN_BS1_9tq  |           9             |
 *        | CAN_BS1_10tq |           10            |
 *        | CAN_BS1_11tq |           11            |
 *        | CAN_BS1_12tq |           12            |
 *        | CAN_BS1_13tq |           13            |
 *        | CAN_BS1_14tq |           14            |
 *        | CAN_BS1_15tq |           15            |
 *        | CAN_BS1_16tq |           16            |
 * @param bs2 传输时间段2,取值范围: 0-7
 *        |  可选参数   | CAN BS2 时间量子         |
 *        | :---------: | :---------------------: |
 *        | CAN_BS2_1tq |            1            |
 *        | CAN_BS2_2tq |            2            |
 *        | CAN_BS2_3tq |            3            |
 *        | CAN_BS2_4tq |            4            |
 *        | CAN_BS2_5tq |            5            |
 *        | CAN_BS2_6tq |            6            |
 *        | CAN_BS2_7tq |            7            |
 *        | CAN_BS2_8tq |            8            |
 * @param sjw 同步跳转宽度,取值范围: 0-3 * bs2 必须大于等于该值 *
 *        |  可选参数   | 同步跳转时间量子宽度 |
 *        | :---------: | :----------: |
 *        | CAN_SJW_1tq |      1       |
 *        | CAN_SJW_2tq |      2       |
 *        | CAN_SJW_3tq |      3       |
 *        | CAN_SJW_4tq |      4       |
 * @param mode
 *        |         可选参数         |               CAN 模块工作模式               |
 *        | :----------------------: | :---------------------------------------: |
 *        |     CAN_Mode_Normal      |                   正常模式                                            |
 *        |    CAN_Mode_LoopBack     | 环回模式\n 发送的数据内部回传,不接收外部数据 |
 *        |     CAN_Mode_Silent      |            静默模式\n 只收不发发                              |
 *        | CAN_Mode_Silent_LoopBack |             静默环回\n 内部收发                                |
 *
 * @retval CAN_InitStatus_Failed 初始化失败
 * @retval CAN_InitStatus_Success 初始化成功
 *
 */
static uint8_t CAN1_init(uint8_t brp, uint8_t bs1, uint8_t bs2, uint8_t sjw,
        uint8_t mode) {
    GPIO_InitTypeDef gpio;
    CAN_InitTypeDef CAN_InitSturcture;

    /// 开启 GPIOB 及功能复用时钟
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO | RCC_APB2Periph_GPIOB, ENABLE);
    /// 开启 CAN1 模块时钟
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_CAN1, ENABLE);

    /// 将 CAN1 外设对就能的 PA11,PA12 重映射到 PB8,PB9上
    GPIO_PinRemapConfig(GPIO_Remap1_CAN1, ENABLE);

    /// 将 CAN1_Rx 引脚 PA8 初始化为浮空输入或带上拉输入
    gpio.GPIO_Pin = GPIO_Pin_8;
    gpio.GPIO_Mode = GPIO_Mode_IPU;
    GPIO_Init(GPIOB, &gpio);

    /// 将 CAN1_Tx 引脚 PA9 初始化为推挽复用输出
    gpio.GPIO_Pin = GPIO_Pin_9;
    gpio.GPIO_Mode = GPIO_Mode_AF_PP;
    gpio.GPIO_Speed = GPIO_Speed_2MHz;
    GPIO_Init(GPIOB, &gpio);

    /** CAN1 波特率及模式初始化
     *  CAN 波特率计算公式
     *  Bps = Fpclk1 / ((bs1+1+bs2+1+1) * brp )
     */
    CAN_StructInit(&CAN_InitSturcture);
//    CAN_InitSturcture.CAN_NART = ENABLE;
    CAN_InitSturcture.CAN_BS1 = bs1;
    CAN_InitSturcture.CAN_BS2 = bs2;
    CAN_InitSturcture.CAN_SJW = sjw;
    CAN_InitSturcture.CAN_Prescaler = brp;
    CAN_InitSturcture.CAN_Mode = mode;

    return CAN_Init(CAN1, &CAN_InitSturcture);
}

static void CAN1_Send(uint8_t CAN_Id_x, uint8_t CAN_RTR_x, uint32_t id,
        char *buf, uint8_t len) {
    CanTxMsg CanTxStructure;
    uint8_t mbox;

    printf("CAN1 send ");
    if (CAN_Id_x == CAN_Id_Standard) {
        id &= 0x7FF;
        CanTxStructure.StdId = id;
        printf("standard ");
    } else {
        CanTxStructure.ExtId = id;
        printf("extend ");
    }

    printf("id(%#010x) ", id);

    CanTxStructure.RTR = CAN_RTR_x;
    if (CAN_RTR_x == CAN_RTR_Data) {
        printf("data: ");
        if (len > 8)
            len = 0;
        CanTxStructure.DLC = len;
        for (uint8_t i = 0; i < len; i++) {
            CanTxStructure.Data[i] = buf[i];
            printf("%02x ", (uint8_t) buf[i]);
        }
    } else {
        printf("remote frame");
    }
    printf("\r\n");

    mbox = CAN_Transmit(CAN1, &CanTxStructure);

    // 通道查询的方式来判断是否发送成功
    if (mbox == CAN_TxStatus_NoMailBox) {
        printf("There is no empty mailbox.\r\n");
    }
#if USE_TX_INTERRUPT != 1
    else {
        int i = 0;
        while ((CAN_TransmitStatus(CAN1, mbox) != CAN_TxStatus_Ok) && (i < 0xFFF))
        {
            i++;
        }

        if (i == 0xFFF) {
            printf("Send can message from %d mailbox failed!\r\n", mbox);
        } else {
            printf("Send can message from %d mailbox success!\r\n", mbox);
        }
    }
#endif
}

int main(int argc, char **argv) {
    char buffer[8] = { 0, 1, 2, 3, 4, 5, 6, 7 };

    System_debug_init(115200);

    // 48M/((6+5+1)*32) = 125Kbps
    CAN1_init(32, CAN_BS1_6tq, CAN_BS2_5tq, CAN_SJW_2tq, CAN_Mode_Normal);

#if USE_TX_INTERRUPT == 1
    NVIC_EnableIRQ(USB_HP_CAN1_TX_IRQn);
    CAN_ITConfig(CAN1, CAN_IT_TME, ENABLE);
#endif

    NVIC_EnableIRQ(USB_HP_CAN1_TX_IRQn);
    NVIC_EnableIRQ(CAN1_SCE_IRQn);
    CAN_ITConfig(CAN1, CAN_IT_ERR | CAN_IT_EPV | CAN_IT_BOF | CAN_IT_TME,
            ENABLE);

//    CAN1_Send(CAN_Id_Standard, CAN_RTR_Remote, 0x812, buffer, 8);

    DeviceState = CAN_DEVICE_STATE_ACTIVE;

    while (1)
    {
        if(DeviceState == CAN_DEVICE_STATE_ACTIVE) {
//        CAN1_Send(CAN_Id_Standard, CAN_RTR_Remote, 0x812, buffer, 8);
            CAN1_Send(CAN_Id_Standard, CAN_RTR_Data, 0x812, buffer, 4);
//        CAN1_Send(CAN_Id_Extended, CAN_RTR_Remote, 0x812, buffer, 8);
//        CAN1_Send(CAN_Id_Extended, CAN_RTR_Data, 0x812, buffer, 8);

            for (uint8_t i = 0; i < 8; i++)
            {
                buffer[i] += 8;
            }
        }

        Delay_Ms(1000);
    }
}

//#if USE_TX_INTERRUPT == 1
__attribute__((interrupt())) void USB_HP_CAN1_TX_IRQHandler() {
    uint32_t esr, ier;
    if ((CAN1->TSTATR & CAN_TSTATR_RQCP0) != 0U) {
        if ((CAN1->TSTATR & CAN_TSTATR_TXOK0) != 0U) {
            printf("CAN1 TX0 complete!\r\n");
            CAN1->TSTATR = CAN_TSTATR_RQCP0; // 清除传输完成标志位
        }
    }
    if ((CAN1->TSTATR & CAN_TSTATR_RQCP1) != 0U) {
        if ((CAN1->TSTATR & CAN_TSTATR_TXOK1) != 0U) {
            printf("CAN1 TX1 complete!\r\n");
            CAN1->TSTATR = CAN_TSTATR_RQCP1; // 清除传输完成标志位
        }
    }
    if ((CAN1->TSTATR & CAN_TSTATR_RQCP2) != 0U) {
        if ((CAN1->TSTATR & CAN_TSTATR_TXOK2) != 0U) {
            printf("CAN1 TX0 complete!\r\n");
            CAN1->TSTATR = CAN_TSTATR_RQCP2; // 清除传输完成标志位
        }
    }

    esr = CAN1->ERRSR;
    ier = CAN1->INTENR;

    if (((esr & CAN_ERRSR_BOFF) == 0U) && ((ier & CAN_INTENR_BOFIE) == 0U)) {
        CAN1->INTENR |= CAN_INTENR_BOFIE;
        printf("CAN1 bus don't busoff!\r\n");
        DeviceState = CAN_DEVICE_STATE_ACTIVE;
    }
    if (((esr & CAN_ERRSR_EPVF) == 0U) && ((ier & CAN_INTENR_EPVIE) == 0U)) {
        CAN1->INTENR |= CAN_INTENR_EPVIE;
        printf("CAN1 bus don't passive!\r\n");
        DeviceState = CAN_DEVICE_STATE_ACTIVE;
    }
    if (((esr & CAN_ERRSR_EWGF) == 0U) && ((ier & CAN_INTENR_EWGIE) == 0U)
            && (((esr & CAN_ERRSR_TEC) >> 16) == 95U)
            && (((esr & CAN_ERRSR_REC) >> 24) < 95U)) {
        CAN1->INTENR |= CAN_INTENR_EWGIE;
    }
}
//#endif

__attribute__((interrupt())) void CAN1_SCE_IRQHandler() {
    uint32_t esr, ier;
    esr = CAN1->ERRSR;
    ier = CAN1->INTENR;
    CAN1->STATR = CAN_STATR_ERRI; // 清除状态寄存器中的错误中断标志
    if (((esr & CAN_ERRSR_BOFF) != 0U) && ((ier & CAN_INTENR_BOFIE) != 0U)) {
        CAN1->INTENR &= ~CAN_INTENR_BOFIE; // 关闭总线离线中断
        DeviceState = CAN_DEVICE_STATE_BUSOFF;
        printf("CAN1 bus off !\r\n");
    } else if (((esr & CAN_ERRSR_EPVF) != 0U)
            && ((ier & CAN_INTENR_EPVIE) != 0U)) {
        CAN1->INTENR &= ~CAN_INTENR_EPVIE; // 关闭被动错误中断
        DeviceState = CAN_DEVICE_STATE_PASSTIVE;
        printf("CAN1 bus passive !\r\n");
    } else if (((esr & CAN_ERRSR_EWGF) != 0U)
            && ((ier & CAN_INTENR_EWGIE) != 0U)) {
        CAN1->INTENR &= ~CAN_INTENR_EWGIE; // 关闭错误预警中断
        printf("CAN1 warring !\r\n");
    }
}

