/**
 * @file usart1_rx_dma_interrupt.c
 * @author TGL(13882149056@qq.com)
 * @brief USART1的接收DMA示例
 * @version 0.1
 * @date 2022-11-19
 * 
 * @copyright Copyright (c) 2022
 * 
 * @details
 * 
 * + 硬件连接
 *   USART1_Tx(PA.9) ===> 串口的Rx引脚
 *   USART1_Rx(PA.10) <=== 串口的Tx引脚
 * + 终端输出
 *       系统时钟：72000000Hz
 *   APB1总线时钟：36000000Hz
 *   APB2总线时钟：72000000Hz
 *   在终端入字符，当输入字符长度大到16字节时输出之前输入的16个字节内容
 *   >>> 1234567890123456
 *   USART1 receive 16 bytes data: 1234567890123456 by DMA1 channel5.
 *   >>> abcdef1234567890
 *   USART1 receive 16 bytes data: abcdef1234567890 by DMA1 channel5.
 *   >>> 
 * 
 */
#include "debug.h"

uint8_t RxBuffer[16] = {0};

void usart1_init(uint32_t baudrate)
{
    USART_InitTypeDef usart;
    GPIO_InitTypeDef gpio;

    // 开启GPIOA引脚及USART1及模块复用功能模块时钟
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_USART1 | RCC_APB2Periph_AFIO, ENABLE);

    // 配置PA.9为复用输出，作为USART1的Tx引脚
    gpio.GPIO_Pin = GPIO_Pin_9;
    gpio.GPIO_Mode = GPIO_Mode_AF_PP;
    gpio.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOA, &gpio);

    // 配置PA.10为带上拉输入，作为USART1的Rx引脚
    gpio.GPIO_Pin = GPIO_Pin_10;
    gpio.GPIO_Mode = GPIO_Mode_IPU;
    GPIO_Init(GPIOA, &gpio);

    // 初始化USART配置参数为：9600，8，n,1 无硬件控制流
    USART_StructInit(&usart);
    // 重新配置波特率
    usart.USART_BaudRate = baudrate;
    // usart1初始化
    USART_Init(USART1, &usart);

    // 开启usart1的接收DMA功能，当接收到数据自动触发对应通道的DMA传输
    USART_DMACmd(USART1, USART_DMAReq_Rx, ENABLE);
    // 同时开启接收中断，在接收中断处理函数中返回接收到的数据。实现输入回显功能
    USART_ITConfig(USART1, USART_IT_RXNE, ENABLE);
    // 开启USART1中断通道，回显示功能使用
    NVIC_EnableIRQ(USART1_IRQn);

    // 启动usart1进行收发数据
    USART_Cmd(USART1, ENABLE);
}

// usart1中断函数申明
void USART1_IRQHandler(void) __attribute__((interrupt()));
void USART1_IRQHandler(void)
{
    // 回传收到的数据
    USART_SendData(USART1, USART1->DATAR);
    // 清接收中断标志
    USART_ClearITPendingBit(USART1, USART_IT_RXNE);
}

void usart1_rx_dma_init(void)
{
    DMA_InitTypeDef dma;

    // 开启DMA1的时钟
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);

    // 复位DMA1的channel5
    DMA_DeInit(DMA1_Channel5);

    DMA_StructInit(&dma);
    // DMA的外设地址为USART1的数据接收寄存地址
    dma.DMA_PeripheralBaseAddr = (uint32_t)(&USART1->DATAR);
    // 外设的数据宽度为1Byte
    dma.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
    // 外设地址不递增
    dma.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
    // 内存在址为RxBuffer数组地址
    dma.DMA_MemoryBaseAddr = (uint32_t)RxBuffer;
    // 内存地址处存储数据的位宽也为1Byte
    dma.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;
    // 内存地址递增
    dma.DMA_PeripheralInc = DMA_MemoryInc_Enable;
    // DMA传输的总数据大小，这儿为16字节
    dma.DMA_BufferSize = sizeof(RxBuffer);
    // DMA传输的方向是外设到内存的传输
    dma.DMA_DIR = DMA_DIR_PeripheralSRC;
    // 不是内存到内存的传输
    dma.DMA_M2M = DMA_M2M_Disable;
    // DMA采用循环模式，也就是16字节传完了，又会从头开始传输
    dma.DMA_Mode = DMA_Mode_Circular;
    // DMA传输优先线为低
    dma.DMA_Priority = DMA_Priority_Low;
    // 初始化DAM1的通道5，该通道对应了usart1的RX传输
    DMA_Init(DMA1_Channel5, &dma);

    // 开启DMA传输功能
    DMA_Cmd(DMA1_Channel5, ENABLE);

    // 当传输完成时产生中断
    DMA_ITConfig(DMA1_Channel5, DMA_IT_TC, ENABLE);
    // 开启DMA中断
    NVIC_EnableIRQ(DMA1_Channel5_IRQn);
}

// DMA中断申明
void DMA1_Channel5_IRQHandler(void) __attribute__((interrupt()));
void DMA1_Channel5_IRQHandler(void)
{
    // 打印收到的全部16字节数据
    printf("\r\nUSART1 receive %d bytes data: %s by DMA1 channel5.\r\n>>> ", sizeof(RxBuffer), RxBuffer);
    fflush(stdout);
    // 清除中断标志
    DMA_ClearITPendingBit(DMA1_IT_TC5);
}

int main(void)
{
    RCC_ClocksTypeDef clocks;

    SystemCoreClockUpdate();
    Delay_Init();
    usart1_init(115200);
    usart1_rx_dma_init();
    RCC_GetClocksFreq(&clocks);

    printf("%16s：%dHz\r\n", "系统时钟", clocks.SYSCLK_Frequency);
    printf("%16s：%dHz\r\n", "APB1总线时钟", clocks.PCLK1_Frequency);
    printf("%16s：%dHz\r\n", "APB2总线时钟", clocks.PCLK2_Frequency);
    printf("在终端入字符，当输入字符长度大到16字节时输出之前输入的16个字节内容\r\n>>> ");
    fflush(stdout);

    while (1) {
    }
}
