#include "debug.h"

typedef struct {
    GPIO_TypeDef *Port; ///< WS2812驱动端口
    uint16_t Pin;       ///< WS2812驱动引脚
    uint32_t RCC_GPIO;  ///< GPIO模块时钟
} WS2812_GpioTypeDef;

uint32_t color[] = {0x00, 0x01, 0x0100, 0x010000};

void WS2812_Gpio_Init(WS2812_GpioTypeDef *ws2812)
{
    GPIO_InitTypeDef gpio;

    RCC_APB2PeriphClockCmd(ws2812->RCC_GPIO, ENABLE);

    gpio.GPIO_Pin = ws2812->Pin;
    gpio.GPIO_Mode = GPIO_Mode_Out_PP;
    gpio.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(ws2812->Port, &gpio);

    // 默认控制引脚为低电平
    GPIO_WriteBit(ws2812->Port, ws2812->Pin, Bit_RESET);
}

/**
 * @brief 实现纳秒级延时
 *
 * 72M的频率下，1条指令延时大约为14纳秒的样子
 *
 * @param count 延时时长
 */
void ws2812_Gpio_Delay(uint32_t count)
{
    while (count--) {
        __NOP();
    }
}

/**
 * @brief 向ws2812写1位数据
 *
 * @param ws2812 @ref WS2812_GpioTypeDef 灯带对应的驱动对象
 * @param bit 该位数据值
 *
 * @details:
 * ws2812高电平波形为： 高电平：580ns~1600ns 低电平：220ns~420ns  正脉宽占空比为：3 : 1 PWM周期不能超过2000ns
 * ws2812低电平波形为： 高电平：220ns~380ns  低电平：580ns~1600ns 正脉宽占空比为：1 ：3 PWM周期不能超过2000ns
 * ws2812复位波形为： 低电平保持280us以上
 */
void ws2812_Gpio_WriteBit(WS2812_GpioTypeDef *ws2812, BitAction bit)
{
    if (bit) {
        GPIO_WriteBit(ws2812->Port, ws2812->Pin, Bit_SET);
        // 试验过6：2和3：1是可以正常工作的，9：3就会出现全高亮。
        ws2812_Gpio_Delay(6);
        GPIO_WriteBit(ws2812->Port, ws2812->Pin, Bit_RESET);
        ws2812_Gpio_Delay(2);
    } else {
        GPIO_WriteBit(ws2812->Port, ws2812->Pin, Bit_SET);
        ws2812_Gpio_Delay(2);
        GPIO_WriteBit(ws2812->Port, ws2812->Pin, Bit_RESET);
        ws2812_Gpio_Delay(6);
    }
}

/**
 * @brief ws2812控制器复位
 *
 * 保持280us以上低电平就会复位
 * @param ws2812
 */
void ws2812_Gpio_Reset(WS2812_GpioTypeDef *ws2812)
{
    GPIO_WriteBit(ws2812->Port, ws2812->Pin, Bit_SET);
    GPIO_WriteBit(ws2812->Port, ws2812->Pin, Bit_RESET);
    ws2812_Gpio_Delay(10);
    //    GPIO_WriteBit(ws2812->Port, ws2812->Pin, Bit_SET);
    //    GPIO_WriteBit(ws2812->Port, ws2812->Pin, Bit_RESET);
}

/**
 * @brief 将RGB颜色数据转换成WS2812使用的GRB数据
 *
 * @param red 红色值，0-0xff
 * @param green 绿色值，0-0xff
 * @param blue 监色值，0-0xff
 * @return uint32_t ws2812使用的颜色值
 */
uint32_t ws2812_RGB2GRB(uint8_t red, uint8_t green, uint8_t blue)
{
    return green << 16 | red << 8 | blue;
}

/**
 * @brief 从高位开始依次发送24位颜色值
 *
 * @param ws2812 led灯带驱动对象
 * @param grb 颜色值
 */
void ws2812_Gpio_WritePixel(WS2812_GpioTypeDef *ws2812, uint32_t grb)
{
    uint32_t temp = grb << 8;
    for (int i = 0; i < 24; i++) {
        ws2812_Gpio_WriteBit(ws2812, temp & 0x80000000);
        temp <<= 1;
    }
}

int main(void)
{
    uint8_t index = 0;

    RCC_ClocksTypeDef clocks;
    // 创建一个GPIO驱动ws2812的对象
    WS2812_GpioTypeDef ws2812 = {GPIOA, GPIO_Pin_0, RCC_APB2Periph_GPIOA};

    Delay_Init();
    USART_Printf_Init(115200);
    SystemCoreClockUpdate();
    RCC_GetClocksFreq(&clocks);

    printf("系统时钟频率: %dHz\r\n", clocks.SYSCLK_Frequency);
    printf("APB1总线时钟频率: %dHz\r\n", clocks.PCLK1_Frequency);
    printf("APB2总线时钟频率: %dHz\r\n", clocks.PCLK2_Frequency);

    printf("PA0初始化为WS2812驱动引脚\r\n");
    WS2812_Gpio_Init(&ws2812);
    printf("将WS2812的DI引脚跟PA0相连！将以绿，红，蓝的顺序点灯6可LED灯\r\n");

    while (1) {
        // 循环2次，如果每次写1个灯哪么就只以点2只灯；如果第次写3个灯哪就点6只灯
        // for (int i = 0; i < 2; i++) {
        //     // ws2812_Gpio_WritePixel(&ws2812, ws2812_RGB2GRB(0x04, 0x04, 0x04));

        //     ws2812_Gpio_WritePixel(&ws2812, 0x01);
        //     ws2812_Gpio_WritePixel(&ws2812, 0x0100);
        //     ws2812_Gpio_WritePixel(&ws2812, 0x010000);
        // }
        
        // 6灯滚动灯效果
        for (int i = 0; i < 6; i++) {
            ws2812_Gpio_WritePixel(&ws2812, color[(index + i) & 0x3]);
        }
        index++;
        // 每次数据发送完后都应发送1个复位信号，从复信号后再写的数据将又从箱1个灯开始写入
        ws2812_Gpio_Reset(&ws2812);

        Delay_Ms(1000);
    }
}
