---
title: GPIO之LED控制
categories: [ch32v307, gpio]
tags: [risc-v]
data: 2022-11-04 15:53:00
author: 唐国林
---

## GPIO一般操作流程

```mermaid

flowchart LR
    A(开启GPIO端口时钟) --> B(配置端口引脚模式) --> c(读写端口对应引脚值)

```

## LED闪烁示例

1. GPIO直驱LED通过延时来实现闪灯
2. 在主循环中先读按键状态，控制LED灯亮与灭
3. 按键配置成中断，在中断处理程序中控制LED亮与灭
4. 配置基础定时器产生1Hz的中断频率，在中断处理函数中闪灯

```c gpio.c
#include "debug.h"

void PA0_1_Init(void)
{
    // 开启GPIOA模块时钟源
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);

    GPIO_InitTypeDef gpio_init;
    // 默认将端口所有引脚初始化为2M模拟量输入
    GPIO_StructInit(&gpio_init);

    gpio_init.GPIO_Mode = GPIO_Mode_Out_PP;
    gpio_init.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_4 | GPIO_Pin_5;
    gpio_init.GPIO_Speed = GPIO_Speed_50MHz;

    // 将PA.0初始化为推免输出
    GPIO_Init(GPIOA, &gpio_init);

    printf("PA.0和PA.1,PA.4,PA.5 初始化为推免输出\r\n");
}

void PA0_Toggle(void)
{
    // PA.0电平翻转
    GPIO_WriteBit(GPIOA, GPIO_Pin_0, !GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_0));
    // 延时500ms
    Delay_Ms(500);
}

void PA2_Init(void)
{
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);

    GPIO_InitTypeDef gpio_init;

    gpio_init.GPIO_Pin = GPIO_Pin_2;
    gpio_init.GPIO_Mode = GPIO_Mode_IPU;

    // PA.2初始化为带下打输入，以读取外部按键状态
    GPIO_Init(GPIOA, &gpio_init);
}

void PA1_LED(void)
{
    if (GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_2) == Bit_RESET) {
        Delay_Ms(50);
        if (GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_2) == Bit_RESET) {
            if (GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_1) == Bit_SET) {
                GPIO_WriteBit(GPIOA, GPIO_Pin_1, Bit_RESET);
                printf("LED ON\r\n");
            } else {
                GPIO_WriteBit(GPIOA, GPIO_Pin_1, Bit_SET);
                printf("LED OFF\r\n");
            }
        }
    }
}

void PA3_INT_Init(void)
{
    GPIO_InitTypeDef gpio_init = {0};
    EXTI_InitTypeDef exti_init = {0};
    NVIC_InitTypeDef nvic_init = {0};

    // 引脚中断输入必须开启引脚和引脚复用两个模块的时钟源
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_AFIO, ENABLE);

    gpio_init.GPIO_Pin = GPIO_Pin_3;
    gpio_init.GPIO_Mode = GPIO_Mode_IPU;
    // PA.3配置为带上拉输入模式
    GPIO_Init(GPIOA, &gpio_init);

    // 将PA端口配置为EXIT3的输入端口
    GPIO_EXTILineConfig(GPIO_PortSourceGPIOA, GPIO_PinSource3);
    // 引脚的端口索引必须跟中断索引对应，PA3只能映射到EXTI_Line3上
    exti_init.EXTI_Line = EXTI_Line3;
    exti_init.EXTI_Mode = EXTI_Mode_Interrupt;
    exti_init.EXTI_Trigger = EXTI_Trigger_Falling;
    exti_init.EXTI_LineCmd = ENABLE;
    // 将外部中断3配置为下降沿中断，对应的外部输入引脚为PA3
    EXTI_Init(&exti_init);

    nvic_init.NVIC_IRQChannel = EXTI3_IRQn;
    nvic_init.NVIC_IRQChannelPreemptionPriority = 1;
    nvic_init.NVIC_IRQChannelSubPriority = 2;
    nvic_init.NVIC_IRQChannelCmd = ENABLE;
    // 中断控制器中开启对应通信的中断功能
    NVIC_Init(&nvic_init);

}

// 中断函数必须添加以下属性，或__attribute__((interrupt("WCH-Interrupt-fast")))
__attribute__((interrupt()))
void EXTI3_IRQHandler(void)
{
    // 清除中断标志
    EXTI_ClearITPendingBit(EXTI_Line3);
    if (GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_3) == Bit_RESET) {
        if (GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_4) == Bit_SET) {
            printf("LED ON\r\n");
            GPIO_WriteBit(GPIOA, GPIO_Pin_4, Bit_RESET);
        } else {
            printf("LED OFF\r\n");
            GPIO_WriteBit(GPIOA, GPIO_Pin_4, Bit_SET);
        }
    }
}

void TIM6_Init(uint16_t arr, uint16_t psc)
{
    TIM_TimeBaseInitTypeDef tim_timebase_init;

    // 开启TIM6基础定时器时钟
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM6, ENABLE);

    // 配置基础定时器周期
    tim_timebase_init.TIM_Period = arr;
    // 配置时钟源分频值
    tim_timebase_init.TIM_Prescaler = psc;
    TIM_TimeBaseInit(TIM6, &tim_timebase_init);

    // 开启重复计数功能
    TIM_ARRPreloadConfig(TIM6, ENABLE);
    // 开启更新中断功能
    TIM_ITConfig(TIM6, TIM_IT_Update, ENABLE);

    // 中断控制器中开启对应的中断通道
    NVIC_EnableIRQ(TIM6_IRQn);
}

// 基础定时器中断处理函数
__attribute__((interrupt("WCH-Interrupt-fast")))
void TIM6_IRQHandler(void)
{
    // 清除中断标志
    TIM_ClearITPendingBit(TIM6, TIM_IT_Update);
    GPIO_WriteBit(GPIOA, GPIO_Pin_5, !GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_5));
}

int main(void)
{
    RCC_ClocksTypeDef clocks;

    Delay_Init();
    USART_Printf_Init(115200);
    SystemCoreClockUpdate();
    RCC_GetClocksFreq(&clocks);

    printf("系统时钟频率: %dHz\r\n", clocks.SYSCLK_Frequency);
    printf("APB1总线时钟频率: %dHz\r\n", clocks.PCLK1_Frequency);
    printf("APB2总线时钟频率: %dHz\r\n", clocks.PCLK2_Frequency);

    PA0_1_Init();
    printf("1. LED跟PA0相连时, 以1秒为周期闪烁\r\n");
    PA2_Init();
    printf("2. 将KEY跟PA2相连，将LED跟PA1相连。以实现按键控灯\r\n");
    PA3_INT_Init();
    printf("3. 将PA3配置为下降沿中断输入引脚,将其跟KEY相连，LED跟PA4相连。以实现中断式按键控灯\r\n");
    // 36M/(35999+1)/1000 = 1Hz的频率，其中正负脉宽各500ms
    TIM6_Init(1000, 35999);
    printf("4. 将LED跟PA5相连，用定时器控制闪灯\r\n");
    TIM_Cmd(TIM6, ENABLE);

    while (1) {
        // PA0输出周期为1s的方波
        PA0_Toggle();
        // 循环按键控灯，跟手性欠佳。中断式跟手性更佳
        PA1_LED();
    }

}

```
